Package: reconsi
Type: Package
Title: Resampling Collapsed Null Distributions for Simultaneous
        Inference
Version: 1.25.0
Authors@R: 
    c(person(given = "Stijn", family = "Hawinkel", 
    email = "stijn.hawinkel@psb.ugent.be", 
    role=c("cre", "aut"), comment = c(ORCID = "0000-0002-4501-5180")))
Description: Improves simultaneous inference under dependence of tests by
    estimating a collapsed null distribution through resampling. Accounting for 
    the dependence between tests increases the power while reducing the
    variability of the false discovery proportion. This dependence is common in
    genomics applications, e.g. when combining flow cytometry measurements with
    microbiome sequence counts. 
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.2.1
Imports: phyloseq, ks, reshape2, ggplot2, stats, methods, graphics,
        grDevices, matrixStats, Matrix
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
biocViews: Metagenomics, Microbiome, MultipleComparison, FlowCytometry
BugReports: https://github.com/CenterForStatistics-UGent/reconsi/issues
LazyData: true
git_url: https://git.bioconductor.org/packages/reconsi
git_branch: devel
git_last_commit: 1f35a62
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:37:14 UTC; biocbuild
Author: Stijn Hawinkel [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-4501-5180>)
Maintainer: Stijn Hawinkel <stijn.hawinkel@psb.ugent.be>

# Generated by roxygen2: do not edit by hand

export(getApproxCovar)
export(getC1prop)
export(plotApproxCovar)
export(plotCovar)
export(plotNull)
export(reconsi)
export(testDAA)
exportMethods(testDAA)
import(ggplot2)
import(methods)
import(phyloseq)
importFrom(Matrix,nearPD)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,image)
importFrom(ks,dkde)
importFrom(ks,kde)
importFrom(ks,pkde)
importFrom(matrixStats,colRanks)
importFrom(matrixStats,weightedMean)
importFrom(reshape2,melt)
importFrom(stats,approx)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,pwilcox)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,smooth.spline)
importFrom(stats,var)

#' Obtain the consensus null
#'
#' @param statObs A vector of lenght p with observed test statistics
#' @param statsPerm  A pxB matrix with permuation z-values
#' @param z0Quant a vector of length of quantiles defining the central part R
#'    of the distribution. If a single number is supplied, then
#'    (z0quant, 1-z0quant) will be used
#' @param gridsize An integer, the gridsize for the density estimation
#' @param maxIter An integer, the maximum number of iterations in determining R
#' @param tol The convergence tolerance.
#' @param estP0args A list of arguments passed on to the estP0args() function
#' @param testPargs A list of arguments passed on to quantileFun
#' @param B an integer, the number of permutations
#' @param p an integer, the number of hypotheses
#' @param pi0 A known fraction of true null hypotheses
#' @param assumeNormal A boolean, should normality be assumed for the null distribution?
#' @param resamAssumeNormal A boolean, should normality be assumed for resampling dists
#' @importFrom stats qnorm dnorm approx quantile
#' @importFrom ks kde dkde
#'
#' @return A list with following entries
#' \item{PermDensFits}{The permutation density fits}
#' \item{zSeq}{The support of the kernel for density estimation}
#' \item{zValsDensObs}{The estimated densities of the observed z-values}
#' \item{convergence}{A boolean, has the algorithm converged?}
#' \item{weights}{Vector of length B with weights
#'    for the permutation distributions}
#' \item{fdr}{Estimated local false discovery rate along the support
#'    of the kernel}
#' \item{p0}{The estimated fraction of true null hypotheses}
#' \item{iter}{The number of iterations}
#' \item{fitAll}{The consensus null fit}
getG0 = function(statObs, statsPerm, z0Quant, gridsize,
                 maxIter, tol, estP0args, testPargs, B, p,
                 pi0, assumeNormal, resamAssumeNormal){
  if(length(statObs)!=nrow(statsPerm)){
    stop("Dimensions of observed and permutation test statistics don't match!")
  }
if(length(z0Quant)==1) {
    z0Quant = sort(c(z0Quant, 1-z0Quant))
}
 estPi0 = is.null(pi0) #Should the fraction of nulls be estimated?
  statObs = statObs[!is.na(statObs)] #ignore NA values
  centralBorders = quantile(statObs, probs = z0Quant)
  #Estimate observed densities
  fitObs = kde(statObs)
  zValsDensObs = dkde(fitObs, x = statObs)
  zValsDensObs[zValsDensObs<=0] = .Machine$double.eps
  #Estimate permutation densities
  if(resamAssumeNormal){
      PermDensFits = apply(statsPerm, 2, estNormal)
      LogPermDensEvals = apply(PermDensFits, 2, function(fit){
          dnorm(statObs, mean = fit[1], sd = fit[2], log = TRUE)
      })
  } else {
    PermDensFits = apply(statsPerm, 2, kde)
    LogPermDensEvals = vapply(PermDensFits, FUN.VALUE = statObs, function(fit){
        log(dkde(fit, x = statObs))
    })
  }
  #Indicators for the observed z values in the support of the kernel
  iter = 1L; convergence = FALSE; p0 = 1
  #Start from standard normal, even when not assumed. It's only a starting value
  fitAll = c("mean" = 0, "sd" = 1)
  g0Obs = dnorm(statObs, mean = fitAll[1], sd = fitAll[2])
  fdr = g0Obs/zValsDensObs*p0
  fdr[fdr>1] = 1 #Normalize here already!
  fdr[fdr==0] = .Machine$double.eps
  if(!assumeNormal){
      resamVec = c(t(statsPerm))
  } else {
      prepVec = colSums(PermDensFits^2)
  }
  while(iter <= maxIter && !convergence){
      fdrOld = fdr; p0old = p0
      weights = calcWeights(logDensPerm = LogPermDensEvals, fdr = fdr)
      #Null distribution
      if(assumeNormal){
        wMean = sum(PermDensFits["mean",]*weights)
        fitAll = c("mean" = wMean,
                   "sd" = sqrt(sum(weights*(prepVec-wMean^2))))
        g0Obs = dnorm(statObs, mean = fitAll[1], sd = fitAll[2])
      } else {
        fitAll = kde(resamVec, w = rep(weights*B,p))
        g0Obs = predict(fitAll, x = statObs)
      }
    fdr = g0Obs/zValsDensObs*p0
    fdr[fdr>1] = 1 #Normalize here already!
    p0 = if(estPi0) do.call(estP0, c(list(statObs = statObs, fitAll = fitAll,
                                          assumeNormal = assumeNormal),
                   estP0args)) else pi0
    convergence = mean((fdr-fdrOld)^2) < tol && (p0-p0old)^2 < tol
    iter = iter + 1L
  }
  if(!convergence){
      warning("Consensus null estimation did not converge, please investigate cause! \n")}
  return(list(PermDensFits = PermDensFits,
              zValsDensObs = zValsDensObs, convergence  = convergence,
              Weights = weights, fdr = fdr,
              p0 = p0, iter = iter, assumeNormal = assumeNormal,
              fitAll = fitAll, fitObs = fitObs, resamAssumeNormal = resamAssumeNormal))
}

#' Find the dependence pat C1 of the approximate covariance matrix, and extract the ratio of the first eigenvalue to the sum of all positive eigenvalues
#'
#' @param statsPerm Matrix of permuted test statistics
#' @param numEig An integer, number of first eigenvalues
#' @param ... passed onto binStats
#' @return A proportion indicating the ratio of the first eigenvalues to the sum of all eigenvalues
#' @export
#' @importFrom Matrix nearPD
#' @examples
#' p = 200; n = 50; B = 5e1
#' x = rep(c(0,1), each = n/2)
#' mat = cbind(
#' matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
#' matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
#' )
#' mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
#' fdrRes = reconsi(mat, x, B = B)
#' getC1prop(fdrRes$statsPerm)
getC1prop = function(statsPerm, numEig = 1, ...){
    bootYs = binStats(statsPerm, ...)
    Ctot = tcrossprod(bootYs-rowMeans(bootYs))/(ncol(statsPerm)-1)
    nu = rowMeans(bootYs)
    C0 = diag(nu)-tcrossprod(nu)/length(statsPerm)
    C1 = Ctot-C0
    eigenValues = nearPD(C1)$eigenvalues #Convert to nearest positive definite matrix, this is sampling error
    sum(eigenValues[seq_len(numEig)])/sum(eigenValues)##The crucial property. this could be used to check the approximation
}

#' Plot an the corvariance matrix of the test statistics estimated through permutations
#' @examples
#' p = 200; n = 50; B = 5e1
#' x = rep(c(0,1), each = n/2)
#' mat = cbind(
#' matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
#' matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
#' )
#' mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
#' fdrRes = reconsi(mat, x, B = B)
#' plotCovar(fdrRes)
#' @export
#' @param reconsiFit The reconsi fit
#' @param col,xlab,ylab,... A list of arguments for the image() function.
#' @importFrom graphics image
#' @importFrom grDevices colorRampPalette
#' @details By default, yellow indicates negative correlaton between test statistics,
#' blue positive correlation
#' @note Note the difference with the plotApproxCovar function, where the
#' covariances between binned test statistics are shown to get an idea between
#' covariances between tail and center values of the univariate null distribution.
#' Here the covariance matrix between all test statistics is shown
#' @return invisible()
#' @seealso \link{plotApproxCovar}
plotCovar = function(reconsiFit, col = colorRampPalette(
    c("yellow","blue"))(12),
    xlab = "Test statistic index", ylab = xlab, ...){
    Seq = seq_len(nrow(reconsiFit$statsPerm))
    image(z = var(t(reconsiFit$statsPerm))[,nrow(reconsiFit$statsPerm):1],
          xlab = xlab, ylab = ylab, col = col, x = Seq, y = Seq, ...)
}

#' A function to obtain a t-test statistic efficiently. For internal use only
#'
#' @param y1,y2 vectors of obsereved values in the two groups
#' @param mm,nn number of observations in the corresponding groups
#'
#' @return A list with items
#' \item{tstat}{The t-test statistic}
#' \item{df}{The degrees of freedom (Welch approximation)}
#'
#' @importFrom stats var
getTstat = function(y1, y2, mm, nn){
  var1 = var(y1)
  var2 = var(y2)
  c(tstat = (mean(y1) - mean(y2))/sqrt(var1/mm + var2/nn),
    df = (var1/mm + var2/nn)^2/((var1/mm)^2/(mm - 1) + (var2/nn)^2/(nn - 1)))
}

context("Faster t-test implementation")

test_that("T-test yields same result as built in version", {
    n = 20
    y1 = rnorm(rpois(1,n)+2)
    y2 = rnorm(rpois(1,n)+2 ,mean = 0.2)
    mm = length(y1)
    nn = length(y2)
    tTestBuiltIn = t.test(y1, y2)
    resultVec = c( tTestBuiltIn[["statistic"]],
                   tTestBuiltIn[["parameter"]])
    names(resultVec) = c("tstat", "df")
    expect_equal(reconsi:::getTstat(y1, y2, mm, nn),
                 resultVec)
})

#' A function to numerically stabilize an exponentiation. For internal use only
#' @param exps the vector to be exponentiated
#'
#' @return the vector with the maximum subtracted
stabExp = function(exps){
    exps-max(exps)
}

#' A function to efficiently row multiply a a-by-b matrix by a vector of
#'  length b. More memory intensive but that does not matter
#'   with given matrix sizes
#' @param matrix a numeric matrix of dimension a-by-b
#' @param vector a numeric vector of length b
#'
#' @details t(t(matrix)*vector) but then faster
#'
#' @return a matrix, row multplied by the vector
rowMultiply = function(matrix, vector){
  matrix * rep(vector, rep(nrow(matrix),length(vector)))
}

#' Microbiomes of Crohn's disease patients and healthy controls
#'
#' Microbiome sequencing data of Crohn's disease patients,
#' and healthy controls, together with other baseline covariates.
#' Both sequencing and flow cytometry data are available.
#'
#' @format A phyloseq object with an OTU-table and sample data
#' \describe{
#'   \item{otu_table}{Count data matrix of 234 taxa in 135 samples}
#'   \item{sample_data}{Data frame of patient covariates}
#' }
#' @source \url{https://www.ncbi.nlm.nih.gov/pubmed/29143816}
"Vandeputte"

#' Obtain a null covariance matrix of binned test statistics
#'
#' @note This is not the covariance matrix of the p test statistic, nor of the
#'    data! It is an approximate covariance matrix of binned test statistics for
#'    visualization and diagnostic purposes.
#'
#' @param statsPerm The pxB matrix of permutation z-values in the columns
#' @param ... passed on to binStats
#'
#' @return The covariance matrix of binned z-values
#' @export
#' @examples
#' p = 200; n = 50; B = 5e1
#' x = rep(c(0,1), each = n/2)
#' mat = cbind(
#' matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
#' matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
#' )
#' mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
#' fdrRes = reconsi(mat, x, B = B)
#' corMat = getApproxCovar(fdrRes$statsPerm)
getApproxCovar = function(statsPerm, ...){
    bootYs = binStats(statsPerm, ...)
  tcrossprod(bootYs-rowMeans(bootYs))/(ncol(statsPerm)-1)
}
#' Bin the test statistic into equally sized bins
#' @param z the matrix of permuted test statistics
#' @param nBins an integer, the number of bins
#' @param binEdges A vector of length 2 with the outer bin edges
#' @return Matrix of binned test statistics
binStats = function(z, nBins = 82L, binEdges = c(-4.1,4.1)){
    Breaks = c(-Inf, seq(binEdges[1], binEdges[2], length.out = nBins+1), Inf)
    binnedZ = apply(z, 2, function(xx){table(cut(xx, breaks = Breaks))})
    return(binnedZ)
}

context("Binned correlation matrix")

test_that("Binned correlation matrix has the right dimensions", {
    n = 20; p = 50; B = 1e2
    mat = matrix(rnorm(n*p), n, p)
    x = sample(c(0,1), n , replace = TRUE)
    nBins = 80L
    reconsiObj = reconsi(Y = mat, x = x)
    expect_equal(dim(reconsi:::getApproxCovar(reconsiObj$statsPerm, nBins = nBins)),
        rep(nBins+2L, 2))
  })

#' Plot the obtained null distribution along with a histogram of observed test
#' statistics
#' @param fit an object returned by the reconsi() (or testDAA()) function
#' @param lowColor,highColor The low and high ends of the colour scale
#' @param idNull indices of known null taxa
#' @param nResampleCurves The number of resampling null distributions to plot
#' @param hSize A double, the size of the line of the collapsed null estimate
#'
#' @return a ggplot2 plot object
#' @import ggplot2
#' @importFrom reshape2 melt
#' @importFrom stats density
#' @importFrom ks dkde
#' @export
#' @examples
#'  p = 180; n = 50; B = 1e2
#'  #Low number of resamples keeps computation time down
#'  x = rep(c(0,1), each = n/2)
#'  mat = cbind(
#'  matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
#'  matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
#'  )
#' #Provide just the matrix and grouping factor, and test using the random null
#' fdrRes = reconsi(mat, x, B = B)
#' plotNull(fdrRes)
plotNull = function(fit, lowColor ="yellow", highColor ="blue",
                    idNull = NULL, nResampleCurves = length(fit$Weights),
                    hSize = 0.5){
    with(fit, {
        zSeq = seq(min(statObs), max(statObs), length.out = 1e3)
        zValsDensPerm = if(resamAssumeNormal){
            apply(PermDensFits, 2, function(Fit){
             dnorm(zSeq, mean = Fit["mean"], sd = Fit["sd"])
            })
            } else {
                vapply(PermDensFits, FUN.VALUE = zSeq, function(Fit){dkde(zSeq, Fit)})
            }
    colnames(zValsDensPerm) = paste0("b", seq_len(ncol(zValsDensPerm)))
    idCurves = Weights >= sort(Weights, decreasing = TRUE)[nResampleCurves]
    #The Curves to plot
    df1 = data.frame(weight = Weights,
                     Curve = colnames(zValsDensPerm))[idCurves,]
    df2 = data.frame(zValsDensPerm[,idCurves], zSeq = zSeq)
    moltdf2 = melt(df2, id.vars = c("zSeq"), variable.name = "Curve",
                   value.name = "Density")
    dfMerged = merge(moltdf2, df1, by = "Curve")
    #Permutation densities
    plot = ggplot(data = dfMerged, aes(x = zSeq, group = Curve, y = Density,
                                col = weight, alpha = weight)) +
        geom_line(linetype = "dashed", size = 0.4) +
        scale_colour_continuous(high = highColor,
                                low = lowColor, name = "Weights") +
        scale_alpha_continuous(guide = FALSE, range = c(0.5,1)) +
        xlab(if(zValues) "z-value" else "Test statistic") +
        ylab("Density/Fdr")  + theme_bw() + xlim(range(statObs))
    #Histogram of observed z-values
    plot = plot + geom_histogram(data = data.frame(statObs = statObs),
                             aes(x = statObs, y = ..density..),
                             inherit.aes = FALSE, bins = 50, alpha = 0.5,
                             fill = "mediumseagreen")
    # Add density functions
    g0 = if(assumeNormal) dnorm(zSeq,  mean = fitAll["mean"], sd = fitAll["sd"]) else dkde(zSeq, fitAll)
    gObs = dkde(zSeq, fitObs)
    gObs[gObs<0]  =.Machine$double.eps
    lfdr = g0/gObs*p0
    lfdr[lfdr>1] = 1
    #Only show lfdr for observed z-values
    lfdr[zSeq > (max(statObs)+0.1) | zSeq < (min(statObs)-0.1)] = NA
    dfDens = data.frame("zSeq" = zSeq, "ResampleNull" = g0,
                        "StandardNormal" = dnorm(zSeq)*sum(g0)/sum(dnorm(zSeq)),
                        "fdr" = lfdr)
    if(!is.null(idNull)){
      #If null taxa known, add true normal density
      nullZdens = estNormal(y = statObs[idNull])
      dfDens$OracleNullDensity = dnorm(zSeq, mean = nullZdens["mean"],
                                 sd = nullZdens["sd"])
    }
    dfDensMolt = melt(dfDens, id.vars = "zSeq", value.name = "density",
                      variable.name = "type")
    plot = plot + geom_line(inherit.aes = FALSE, data = dfDensMolt,
                  aes(x = zSeq, y = density, group = type,
                      linetype = type, size = type)) +
        scale_linetype_manual(name = "",
                              values = c("solid", "dashed", "dotdash", if(!is.null(idNull)) "twodash")) +
        scale_size_manual(values = c(0.2, 0.4, 0.4, if(!is.null(idNull)) 0.3), guide = FALSE)
    # Add red dots for Fdr estimates
    dfFdr = data.frame(statObs = statObs, Fdr = Fdr)
    plot = plot + geom_point(inherit.aes = FALSE, data = dfFdr,
                   aes(x = statObs, y = Fdr), col = "red", size = hSize)
    return(plot)
})
}

context("return of plotNull function")

n = 20; p = 200; B = 1e2
mat = matrix(rnorm(n*p), n, p)
x = sample(c(0,1), n , replace = TRUE)
idDA = seq_len(10) %in% seq_len(p)
mat[x==0, idDA] = mat[x==0, seq_len(10)] + 0.5
reco  = reconsi(Y = mat, x = x)
test_that("testDAA deals with all zero rows", {
    expect_s3_class(plotNull(reco), "ggplot")
    expect_s3_class(plotNull(reco, idNull = !idDA), "ggplot")
})

#' A function to calculate observed and permuation z-statistics
#' on a n-by-p matrix of observations
#'
#' @param Y The nxp data matrix
#' @param center a boolean, should data be centered prior to permuation
#' @param scale a boolean, should data be scaled prior to resampling
#' @param test A function name, possibly user defined. See details.
#' @param x A vector defining the groups. Will be coerced to factor.
#' @param B an integer, the number of permuations
#' @param argList A list of further arguments passed on to the test function
#' @param tieBreakRan A boolean, should ties of permutation test statistics
#'  be broken randomly? If not, midranks are used
#' @param replace A boolean. If FALSE, samples are permuted
#' (resampled without replacement), if TRUE the samples are bootstrapped
#' (resampled with replacement)
#'
#' @return A list with components
#' \item{statObs}{A vector of length p of observed test statistics}
#' \item{statsPerm}{A p-by-B matrix of permutation test statistics}
#' \item{resamDesign}{The resampling design}
#' @importFrom stats runif
#' @importFrom matrixStats colRanks
#'
#' @details For test "wilcox.test" and "t.test",
#' fast custom implementations are used. Other functions can be supplied
#' but must accept a y outcome variable, a x as grouping variable, and possibly
#' a list of other arguments. It must return all arguments needed to evaluate
#' its quantile function if z-values are to be used.
getTestStats = function(Y, center, test = "wilcox.test", x, B,
                        argList, tieBreakRan, replace, scale){
  #Sample size
  n = nrow(Y)
  #enumerate B ways to permute/combine
  resamDesign = if(replace) matrix(ceiling(n*runif(n*B)),
                                  nrow = n, ncol = B) else
      vapply(integer(B), FUN.VALUE = integer(n),function(x) sample.int(n = n))
  if(center){
    if(replace){
      Ycenter = scale(Y, center = TRUE, scale = scale)
  } else {
      Ycenter = Y
        for (ii in unique(x)){
            Ycenter[x==ii,] = scale(Ycenter[x==ii,], center = TRUE, scale = scale)
        }
  }
  } else {
      Ycenter = Y
  }
  if(is.character(test) && (test %in% c("wilcox.test","t.test"))){
    xLog = x==names(table(x))[1]
    nn = table(x)[2]
    mm = table(x)[1]
  if(test=="wilcox.test"){ #Shortcuts possbile in case of Wilcoxon test
    nFac = mm*(mm + 1)/2
    YRanked = colRanks(Y, preserveShape = TRUE, ties.method = "average")
    # TIES <- apply(Yranked, 2, function(r) length(r) != length(unique(r)))
    # NTIES <- apply(Yranked[, TIES], 2, table)
    # SIGMA <- sqrt((mm * nn/12) * ((mm + nn + 1) -
    #                                   sum(NTIES^3 - NTIES)/((mm + nn) * (mm + nn - 1))))
    # CORRECTION <-  sign(statObs[TIES]) * 0.5
    # statObs[TIES] <- (statObs[TIES] - CORRECTION)/SIGMA

    #Observed test statistic
    statObs = colSums(YRanked[xLog,]) - nFac
    #Permuation test statistics
    mmSeries = seq_len(mm)
    statsPerm = - nFac + vapply(seq_len(B), function(ii){
      if(tieBreakRan){
          YRanked = colRanks(Y, ties.method = "random", preserveShape = TRUE)
      #Random tiebreaking, important to combat test statistic discreteness
      }
        colSums(YRanked[resamDesign[mmSeries,ii],])
    }, FUN.VALUE = statObs)



  } else if(test == "t.test"){
    statObs = apply(Y, 2, function(dat){
getTstat(y1 = dat[xLog], y2 = dat[!xLog], mm = mm, nn = nn)
    })
    statsPerm = vapply(seq_len(B), FUN.VALUE = statObs,
                       function(ii){
      xSam = xLog[resamDesign[,ii]]
      apply(Ycenter, 2, function(dat){
        getTstat(y1 = dat[xSam], y2 = dat[!xSam], mm = mm, nn = nn)
      })
    })
  }
    } else {
  testFun = match.fun(test)
  statObs = apply(Y,2, function(y){
      do.call(testFun, c(list(y = y, x = x), argList))
  })
  statsPerm = vapply(seq_len(B),
                     FUN.VALUE = statObs,
                     function(ii){
    apply(Ycenter[resamDesign[,ii],],2, function(y){
    do.call(testFun, c(list(y = y, x = x), argList))
  })})
    }
  return(list(statObs = statObs, statsPerm = statsPerm, resamDesign = resamDesign))
}

context("Faster Wilcoxon rank sum test implementation")

test_that("Wilcoxon rank sum test yields same result as built in version", {
    n = 20; p = 50
    mat = matrix(rnorm(n*p), n, p)
    x = sample(c(0,1), n , replace = TRUE)
    expect_equal(reconsi:::getTestStats(mat, replace = FALSE, center = FALSE,
                                         tieBreakRan = FALSE, x = x, B = 2L)$statObs,
                 apply(mat, 2, function(y){
                     wilcox.test(y~x)$statistic
                    }))
})

#' Plot an approximation of the correlation structure of the test statistics
#' @examples
#' p = 200; n = 50; B = 5e1
#' x = rep(c(0,1), each = n/2)
#' mat = cbind(
#' matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
#' matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
#' )
#' mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
#' fdrRes = reconsi(mat, x, B = B)
#' plotApproxCovar(fdrRes)
#' @export
#' @param reconsiFit The reconsi fit
#' @param col,x,y,xlab,ylab,... A list of arguments for the image() function.
#' @param nBins,binEdges passed on to the getApproxCovar function
#' @importFrom graphics image
#' @importFrom grDevices colorRampPalette
#' @details By default, yellow indicates negative correlaton between bin counts,
#' blue positive correlation
#' @note This is not the covariance matrix of the p test statistic, nor of the
#'    data! It is an approximate covariance matrix of binned test statistics for
#'    visualization purposes. See \link{plotCovar} for the full covariance matrix.
#' @return invisible()
#' @seealso \link{plotCovar}, \link{getApproxCovar}
plotApproxCovar = function(reconsiFit, col = colorRampPalette(
                         c("yellow","blue"))(12),
                         x = seq(-4.2, 4.2, 0.1),
                         y = seq(-4.2, 4.2, 0.1),
                         xlab = "Z-values", ylab = "Z-values",
                         nBins = 82L, binEdges = c(-4.1,4.1), ...){
    image(z = getApproxCovar(reconsiFit$statsPerm,
                       nBins = nBins, binEdges = binEdges),
          x = x, y = y, xlab = xlab, ylab = ylab, col = col, ...)
}

#'Estimate the fraction of true null hypotheses.
#'
#' @param statObs A vector of observed z-values
#' @param fitAll the estimated normal null
#' @param z0quantRange a number of quantiles between 0 and 0.5
#' @param smooth.df degrees of freedom for the spline smoother
#' @param evalVal the value of q at which to evaluate the spline
#' @param assumeNormal A boolean, should normality be assumed for the null distribution?
#'
#' @return The estimated null fraction, the value of the spline evaluated
#'  at the first element of z0quantRange
#'
#' @details A natural spline is used over a range of intervals.
#' Based on the qvalue::qvalue() function and
#'Storey and Tibshirani, 2003
#'
#'@importFrom stats smooth.spline predict
#'@importFrom ks pkde
estP0 = function(statObs, fitAll, z0quantRange, smooth.df, evalVal, assumeNormal){
    pi0 = vapply(z0quantRange, FUN.VALUE = double(1), function(z0Quant) {
    z0Quant = c(z0Quant, 1-z0Quant)
    if(assumeNormal) {
        centralBorders = qnorm(z0Quant, mean = fitAll["mean"], sd = fitAll["sd"])
    } else {
        cumul.prob <- pkde(q = fitAll$eval.points, fhat = fitAll)
        Ord = order(cumul.prob)
        ind <- findInterval(x = z0Quant, vec = cumul.prob[Ord])
        centralBorders = fitAll$eval.points[Ord][ind]
    }
    mean(statObs <= centralBorders[2] &
        statObs >= centralBorders[1])/diff(z0Quant)
    })
    smoothPi0 = smooth.spline(z0quantRange, pi0, df = smooth.df)
    pi0Smooth = predict(smoothPi0, x = evalVal)$y #Conservative
    min(pi0Smooth, 1)
}

## ----installBioConductor, eval = FALSE----------------------------------------
# library(BiocManager)
# BiocManager::install("reconsi")

## ----installAndLoadGitHub, eval = FALSE---------------------------------------
# library(devtools)
# install_github("CenterForStatistics-UGent/reconsi")

## ----loadReconsi--------------------------------------------------------------
suppressPackageStartupMessages(library(reconsi))
cat("reconsi package version", as.character(packageVersion("reconsi")), "\n")

## ----syntData-----------------------------------------------------------------
#Create some synthetic data with 90% true null hypothesis
 p = 200; n = 50
x = rep(c(0,1), each = n/2)
 mat = cbind(
 matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
 matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
 )
 #Provide just the matrix and grouping factor, and test using the collapsed null
 fdrRes = reconsi(mat, x)
  #The estimated tail-area false discovery rates.
  estFdr = fdrRes$Fdr

## ----p0-----------------------------------------------------------------------
fdrRes$p0

## ----plotNull-----------------------------------------------------------------
plotNull(fdrRes)

## ----plotApproxCovar----------------------------------------------------------
plotApproxCovar(fdrRes)

## ----plotCovar----------------------------------------------------------------
matBlock = mat
matBlock[x==0,] = matBlock[x==0,] + rep(c(-1,2), each = n*p/4)
fdrResBlock = reconsi(matBlock, x)
plotCovar(fdrResBlock)

## ----customFunction-----------------------------------------------------------
 #With a custom function, here linear regression
fdrResLm = reconsi(mat, x, B = 5e1,
                      test = function(x, y){
fit = lm(y~x)
c(summary(fit)$coef["x","t value"], fit$df.residual)},
distFun = function(q){pt(q = q[1], df = q[2])})

## ----customFunction2----------------------------------------------------------
 #3 groups
 p = 100; n = 60
x = rep(c(0,1,2), each = n/3)
mu0 = 5
 mat = cbind(
 matrix(rnorm(n*p/10, mean = mu0+x),n,p/10), #DA
 matrix(rnorm(n*p*9/10, mean = mu0),n,p*9/10) #Non DA
 )
 #Provide an additional covariate through the 'argList' argument
 z = rpois(n , lambda = 2)
 fdrResLmZ = reconsi(mat, x, B = 5e1,
 test = function(x, y, z){
 fit = lm(y~x+z)
 c(summary(fit)$coef["x","t value"], fit$df.residual)},
distFun = function(q){pt(q = q[1], df = q[2])},
 argList = list(z = z))

## ----kruskal------------------------------------------------------------------
fdrResKruskal = reconsi(mat, x, B = 5e1,
test = function(x, y){kruskal.test(y~x)$statistic}, zValues = FALSE)

## ----resamZvals---------------------------------------------------------------
fdrResKruskalPerm = reconsi(mat, x, B = 5e1,
test = function(x, y){
 kruskal.test(y~x)$statistic}, resamZvals = TRUE)

## ----bootstrap----------------------------------------------------------------
fdrResBootstrap = reconsi(Y = mat, B = 5e1, test = function(y, x, mu){
                                      testRes = t.test(y, mu = mu)
                                      c(testRes$statistic, testRes$parameter)}, argList = list(mu = mu0),
                                  distFun = function(q){pt(q = q[1],
                                                           df = q[2])})

## ----Vandeputte---------------------------------------------------------------
#The grouping and flow cytometry variables are present in the phyloseq object, they only need to be called by their name.
data("VandeputteData")
testVanDePutte = testDAA(Vandeputte, groupName = "Health.status", FCname = "absCountFrozen", B = 1e2L)

## ----vandeputteFdr------------------------------------------------------------
FdrVDP = testVanDePutte$Fdr
quantile(FdrVDP)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' A function to test for differential absolute abundance on a phyloseq object
#' @param Y A phyloseq object, or a data matrix with samples in the rows and
#'OTUs in the columns
#' @param groupName A character string, the name of a variable in physeq
#'indicating the grouping factor
#' @param FCname A character string, the name of a variable in physeq
#'containing the total cell count
#' @param FC a vector of length n with total flow cytometry cell counts
#' @param x a grouping factor of length n
#' @param S a vector of library sizes. Will be calculated if not provided
#' @param assumeNormal A boolean, should normality be assumed for the null distribution?
#' @param tieBreakRan A boolean, should ties be broken at random.
#' @param ... passed on to the reconsi() function
#'
#'@return See the reconsi() function

#' @rdname testDAA
#' @export
setGeneric("testDAA", function(Y, ...) standardGeneric("testDAA"))

#'@export
#'@import phyloseq
#'@import methods
#'@rdname testDAA
#'@examples
#'#Test for phyloseq object
#'library(phyloseq)
#'data("VandeputteData")
#'VandeputtePruned = prune_samples(Vandeputte,
#'samples = sample_names(Vandeputte)[20:40])
#'testVanDePutte = testDAA(VandeputtePruned, "Health.status", "absCountFrozen",
#'B = 15)
setMethod("testDAA", "phyloseq", function(Y, groupName, FCname, ...){
otuTab  = if(taxa_are_rows(Y)) t(as(otu_table(Y), "matrix")) else
    as(otu_table(Y), "matrix")
testDAA(Y = otuTab, FC = get_variable(Y, FCname),
        x = factor(get_variable(Y, groupName)), ...)
})
#' @export
#' @rdname testDAA
#' @examples
#' #Test for matrix
#' testMat = testDAA(as(otu_table(VandeputtePruned), "matrix"),
#' get_variable(VandeputtePruned, "Health.status"),
#' get_variable(VandeputtePruned,"absCountFrozen"), B = 15)
setMethod("testDAA", "matrix", function(Y, FC, x, S = rowSums(Y),
                                        tieBreakRan = TRUE, assumeNormal = TRUE,...){
    stopifnot(length(FC)==length(x), length(FC)==length(S),
              length(FC)==nrow(Y), is.logical(tieBreakRan))
    idSam = S>0
    if(min(table(x[idSam]))<2L){
        return(NULL)
        }
    reconsi(Y[idSam,colSums(Y)>0]/S[idSam]*FC[idSam], x=x[idSam],
            tieBreakRan = tieBreakRan, assumeNormal = assumeNormal,...)
})

#' Obtain weights as posterior probabilities to calculate the consensus null
#'
#' @param logDensPerm A matrix with B rows  of logged density estimates of the B
#'    permutation distributions, and p columns for the p observed test
#'    statistics
#' @param fdr A vector of local false discovery rates for the observed tests
#'  statistics of length p
#'
#' @return A vector of weights of length B
calcWeights = function(logDensPerm, fdr){
    weights = exp(stabExp(colSums(logDensPerm*fdr)))
    weights/sum(weights)
}

#' Correct quantiles by not returning 0 or 1
#' @param quants A vector of quantiles
#' @return The same vector of quantiles but without 0 or 1 values
quantCorrect = function(quants){
    quants[quants==1] = 1-.Machine$double.eps
    quants[quants==0] = .Machine$double.eps
    return(quants)
}

#' Calculate tail-area (Fdr) and local (fdr) false discovery rates,
#'    based on a certain null distribution
#'
#' @param statObs Vector of observed z-values
#' @param fitAll The parameters of the estimated random null
#' @param fdr local false discovery rate, already estimated
#' @param p the number of hypotheses
#' @param fitObs The kernel density estimate object of all test statistics
#' @param p0 The estimated fraction of null hypotheses
#' @param zValsDensObs estimated densities of observed test statistics
#' @param smoothObs A boolean, should estimated observed densities of the test
#' statistics be used in estimating the Fdr
#' @param assumeNormal A boolean, should normality be assumed for the null distribution?
#' @param ... more arguments, ignored
#'
#' @importFrom stats ecdf approx
#'
#' @return A list with components
#' \item{Fdr}{Tail are false discovery rate}
#' \item{fdr}{Local false discovery rate}
getFdr = function(statObs, fitAll, fdr, p, p0, zValsDensObs, smoothObs,
                  assumeNormal, fitObs, ...)
{
  statObsNotNA = statObs[!is.na(statObs)]
  #Null
  if(assumeNormal){
      G0 = pnorm(statObsNotNA, mean = fitAll["mean"], sd = fitAll["sd"])
      G0[G0>0.5] = pnorm(statObsNotNA[G0>0.5], mean = fitAll["mean"],
                         sd = fitAll["sd"], lower.tail = FALSE)
  } else {
      G0 = pkde(statObsNotNA, fitAll)
      G0[G0>0.5] = 1-G0[G0>0.5]
  }
  #Observed
  zcum = if(smoothObs) {
      pkde(statObsNotNA, fitObs)
  } else {
       ecdf(statObsNotNA)(statObsNotNA)
      }
  zcum[zcum>0.5] = 1-zcum[zcum>0.5]
  #Fdr
  Fdr = G0/zcum*p0
  Fdr[Fdr>1] = 1
  #fdr
  if(is.null(fdr)){
      g0 = if(assumeNormal){
          dnorm(statObsNotNA, mean = fitAll["mean"], sd = fitAll["sd"])
      } else {
          dkde(statObsNotNA, fitAll)
      }
    fdr  = g0/zValsDensObs*p0
    fdr[fdr>1] = 1
  }
  FdrOut = fdrOut = statObs
  FdrOut[!is.na(statObs)] = Fdr
  fdrOut[!is.na(statObs)] = fdr
return(list(Fdr = FdrOut, fdr = fdrOut))
}

#' Perform simultaneous inference through
#' collapsed resampling null distributions
#' @param Y the matrix of sequencing counts
#' @param x a grouping factor. If provided, this grouping factor is permuted.
#' Otherwise a bootstrap procedure is performed
#' @param B the number of resampling instances
#' @param test Character string, giving the name of the function
#'  to test for differential absolute abundance.
#'  Must accept the formula interface. Features with tests resulting in
#'  observed NA test statistics will be discarded
#' @param argList A list of arguments, passed on to the testing function
#' @param distFun the distribution function of the test statistic, or its name.
#' Must at least accept an argument named 'q', 'p' and 'x' respectively.
#' @param zValues A boolean, should test statistics be converted to z-values.
#' See details
#' @param testPargs A list of arguments passed on to distFun
#' @param z0Quant A vector of length 2 of quantiles of the null distribution,
#'    in between which only null values are expected
#' @param gridsize The number of bins for the kernel density estimates
#' @param maxIter An integer, the maximum number of iterations in the estimation
#'    of the null distribution
#' @param tol The tolerance for the infinity norm of the central borders
#'    in the iterative procedure
#' @param center A boolean, should observations be centered
#'    in each group prior to permuations? See details.
#' @param scale a boolean, should data be scaled prior to resampling
#' @param replace A boolean. Should resampling occur with replacement (boostrap)
#' or without replacement (permutation)
#' @param assumeNormal A boolean, should normality be assumed for the null distribution?
#' @param zVals An optional list of observed (statObs) and
#' resampling (statsPerm) z-values. If supplied, the calculation
#'    of the observed and resampling test statistics is skipped
#'    and the algorithm proceeds with calculation
#'    of the consensus null distribution
#' @param estP0args A list of arguments passed on to the estP0 function
#' @param resamZvals A boolean, should resampling rather than theoretical null
#' distributions be used?
#' @param smoothObs A boolean, should the fitted rather than estimated observed
#' distribution be used in the Fdr calculation?
#' @param tieBreakRan A boolean, should ties of resampling test statistics
#'  be broken randomly? If not, midranks are used
#' @param pi0 A known fraction of true null hypotheses. If provided,
#' the fraction of true null hypotheses will not be estimated.
#' Mainly for oracle purposes.
#' @param resamAssumeNormal A boolean, should normality be assumed for resampling dists
#'@details Efron (2007) centers the observations in each group prior
#'  to permutation. As permutations will remove any genuine group differences
#'   anyway, we skip this step by default. If zValues = FALSE,
#'   the density is fitted on the original test statistics rather than converted
#'   to z-values. This unlocks the procedure for test statistics with unknown
#'   distributions, but may be numerically less stable.
#' @return A list with entries
#' \item{statsPerm}{Resampling Z-values}
#' \item{statObs}{Observed Z-values}
#' \item{distFun}{Density, distribution and
#' quantile function as given}
#' \item{testPargs}{Same as given}
#' \item{zValues}{A boolean, were z-values used?}
#' \item{resamZvals}{A boolean, were the resampling null distribution used?}
#' \item{cdfValObs}{Cumulative distribution function evaluation
#' of observed test statistics}
#' \item{p0estimated}{A boolean, was the fraction of true null hypotheses
#' estimated from the data?}
#' \item{Fdr,fdr}{Estimates of tail-area and local false discovery rates}
#' \item{p0}{Estimated or supplied fraction of true null hypotheses}
#' \item{iter}{Number of iterations executed}
#' \item{fitAll}{Mean and standard deviation estimated collapsed null}
#' \item{PermDensFits}{Mean and standard deviations of resamples}
#' \item{convergence}{A boolean, did the iterative algorithm converge?}
#' \item{zSeq}{Basis for the evaluation of the densities}
#' \item{weights}{weights of the resampling distributions}
#' \item{zValsDensObs}{Estimated overall densities, evaluated in zSeq}
#' @export
#' @importFrom stats pnorm qnorm pwilcox
#' @note Ideally, it would be better to only use unique resamples, to avoid
#' unnecesarry replicated calculations of the same test statistics. Yet this
#' issue is almost alwyas ignored in practice; as the sample size grows it also
#' becomes irrelevant. Notice also that this would require to place weights in
#' case of the bootstrap, as some bootstrap samples are more likely than others.
#' @examples
#' #Important notice: low number of resamples B necessary to keep
#' # computation time down, but not recommended. Pray set B at 200 or higher.
#' p = 60; n = 20; B = 5e1
#' x = rep(c(0,1), each = n/2)
#' mat = cbind(
#' matrix(rnorm(n*p/10, mean = 5+x), n, p/10), #DA
#' matrix(rnorm(n*p*9/10, mean = 5), n, p*9/10) #Non DA
#' )
#' fdrRes = reconsi(mat, x, B = B)
#' fdrRes$p0
#' #Indeed close to 0.9
#' estFdr = fdrRes$Fdr
#' #The estimated tail-area false discovery rates.
#'
#' #With another type of test. Need to supply quantile function in this case
#' fdrResLm = reconsi(mat, x, B = B,
#' test = function(x, y){
#' fit = lm(y~x)
#' c(summary(fit)$coef["x","t value"], fit$df.residual)},
#' distFun = function(q){pt(q = q[1], df = q[2])})
#'
#' #With a test statistic without known null distribution(for small samples)
#' fdrResKruskal = reconsi(mat, x, B = B,
#' test = function(x, y){
#' kruskal.test(y~x)$statistic}, zValues = FALSE)
#'
#' #Provide an additional covariate through the 'argList' argument
#' z = rpois(n , lambda = 2)
#' fdrResLmZ = reconsi(mat, x, B = B,
#' test = function(x, y, z){
#' fit = lm(y~x+z)
#' c(summary(fit)$coef["x","t value"], fit$df.residual)},
#' distFun = function(q){pt(q = q[1], df = q[2])},
#' argList = list(z = z))
#'
#' #When nog grouping variable is provided, a bootstrap is performed
#' matBoot = cbind(
#' matrix(rnorm(n*p/10, mean = 1), n, p/10), #DA
#' matrix(rnorm(n*p*9/10, mean = 0), n, p*9/10) #Non DA
#' )
#' fdrResBoot = reconsi(matBoot, B = B,
#' test = function(y, x){testRes = t.test(y, mu = 0, var.equal = TRUE);
#' c(testRes$statistic, testRes$parameter)},
#' distFun = function(q){pt(q = q[1], df = q[2])},
#' center = TRUE, replace = TRUE)
reconsi = function(Y, x = NULL, B = 1e3L, test = "wilcox.test",
                   argList = list(), distFun ="pnorm",
                      zValues = TRUE, testPargs = list(),
                      z0Quant = 0.25, gridsize = 801L,
                      maxIter = 100L, tol = 1e-6, zVals = NULL,
                   center = FALSE, replace = is.null(x), assumeNormal = TRUE,
                    estP0args = list(z0quantRange = seq(0.05,0.45, 0.0125),
                                       smooth.df = 3, evalVal = 0.05), resamZvals = FALSE,
                      smoothObs = TRUE, scale = FALSE,
                      tieBreakRan = FALSE, pi0 = NULL, resamAssumeNormal = TRUE){
    #Basic checks
    stopifnot(is.matrix(Y), is.list(argList), is.logical(center),
              is.logical(smoothObs),
              is.logical(resamZvals), is.logical(replace), is.numeric(z0Quant),
              is.numeric(tol), is.numeric(maxIter), is.numeric(gridsize),
              is.numeric(B), is.list(estP0args))
    if(is.function(test)){

    }
    else if(is.character(test)){
        if(test %in% c("wilcox.test","t.test")){
            if(is.null(x) || NCOL(x)!=1){
                stop("Provide single grouping variable for Wilcoxon rank sum test or t-test")
            }
            x = factor(x)
            if(nlevels(x)>2){stop("Wilcoxon rank sum test and t-test only apply to two groups! \n Try 'kruskal.test()' or 'lm()'.")}
        }
      if(test == "t.test"){
        distFun = "ptEdit"
        if(!zValues) {
            stop("With t-tests, the test statistic must be converted to zValues. Please set zValues = TRUE")
        }
        } else if(test=="wilcox.test"){
            if(center) stop("Only two-sample Wilcoxon test natively implemented. provide custom test function!\n")
        testPargs = list(m = table(x)[1], n = table(x)[2])
        distFun = function(q, m, n){
            id = q > (m*n/2)
            q[id] = pwilcox(q[id] - 1, m, n)
            q[!id] = pwilcox(q[!id], m, n)
            q
        }
        }
        }
    if(!"q" %in% names(formals(distFun))){
        stop("Distribution function must accept arguments named 'q'\n")
    }
    if(!"y" %in% names(formals(test)) && !test %in% c("t.test", "wilcox.test")){
        stop("Test function must accept 'y' argument\n")
    }
    if(!is.matrix(Y)){
        stop("Please provide a data matrix as imput!\n",
               if(is.vector(Y)) "Multiplicity correction only needed when testing multiple hypotheses.")
    }
    if(!is.null(x) && NROW(x)!=nrow(Y)){
        stop("Length of grouping factor does not correspond to number of rows of data matrix!\n")
    }
    if(ncol(Y)<30){
     warning("Less than 30 hypotheses tested, multplicity correction may be unreliable.\nConsider using p.adjust(..., method ='BH').",
             immediate. = TRUE)
    }
    if(nrow(Y)<7){
        warning("Small sample size may not yield sufficient unique resampling instances.\nConsider using p.adjust(..., method ='BH').",
                immediate. = TRUE)
    }
      p = ncol(Y)
    if(is.null(zVals)){
    #Test statistics
    testStats = getTestStats(Y = Y, center = center, test = test,
                             x = x, B = B, argList = argList, scale = scale,
                             tieBreakRan = tieBreakRan, replace = replace)
    #Observed cdf values
    cdfValObs = apply(matrix(testStats$statObs, ncol = p), 2, function(stats){
        quantCorrect(do.call(distFun, c(list(q = stats), testPargs)))
    })
    if(zValues){
    #If procedure works with z-values rather than raw test statistics,
        #convert to z-values
    #Resample cdf-values
    cdfValsMat =  apply(testStats$statsPerm,
                      MARGIN = if(length(dim(testStats$statsPerm))==3) c(2,3)
                      else 2, function(stats){
                          quantCorrect(do.call(distFun,
                                               c(list(q = stats), testPargs)))
                      })
    #Observed z-values
    statObs = qnorm(
        if(resamZvals) quantCorrect(vapply(seq_along(cdfValObs),
                                          FUN.VALUE = double(1),
                                          function(i){
            (sum(cdfValObs[i] > cdfValsMat[i,])+1L)/(B+2L)
        })) else cdfValObs
    )
    #Resample z-values
    statsPerm = qnorm(
        if(resamZvals) vapply(seq_len(B), FUN.VALUE = statObs, function(b){
            quantCorrect(vapply(seq_along(cdfValObs), FUN.VALUE = double(1),
                                function(i){
                (sum(cdfValsMat[i,b] > cdfValsMat[i,-b])+1L)/(B+2L)
            }))
        }) else cdfValsMat
    )
    } else {
        #If procedure works on raw test statistics, not many conversions are needed
    #Observed statistics
    statObs = if(is.matrix(testStats$statObs)) testStats$statObs[1,] else
      testStats$statObs
    #Permuation statistics
    statsPerm = if(length(dim(testStats$statsPerm))==3)
        testStats$statsPerm[1,,] else testStats$statsPerm
    }
    } else {
        #If test statistics give, recover them
      statObs = zVals$statObs; statsPerm = zVals$statsPerm
      cdfValObs = zVals$cdfValObs
     }
      if(zValues) {
          testPargs = list()
          }
    #Consensus distribution estimation
    consensus = getG0(statObs = statObs, statsPerm =  statsPerm,
                      z0Quant = z0Quant, gridsize = gridsize, maxIter = maxIter,
                      tol = tol, estP0args = estP0args, testPargs = testPargs,
                      B = B, p = p, pi0 = pi0, assumeNormal = assumeNormal,
                      resamAssumeNormal = resamAssumeNormal)
    #False discovery Rates
    FdrList = do.call(getFdr,
                      c(list(statObs = statObs, p = p, smoothObs = smoothObs),
                        consensus))
    consensus$fdr = NULL
    names(statObs) = names(FdrList$Fdr) = names(FdrList$fdr) = colnames(Y)
    c(list(statsPerm = statsPerm, statObs = statObs, zValues = zValues,
           resamZvals = resamZvals, cdfValObs = cdfValObs,
           testPargs = testPargs, distFun = distFun,
           p0estimated = is.null(pi0),
           resamDesign = if(is.null(zVals)) testStats$resamDesign),
      FdrList, consensus)
}

context("Weights calculation")

test_that("Weights sum to one",{
    n = 20; p = 50; B = 1e2
    mat = matrix(rnorm(n*p), n, p)
    x = sample(c(0,1), n , replace = TRUE)
    expect_equal(sum(reconsi(mat, x)$Weights), 1)
})

context("input to reconsi")

n = 20; p = 50; B = 1e2
mat = matrix(rnorm(n*p), n, p)
x = sample(c(0,1), n , replace = TRUE)

test_that("reconsi throws errors to wrong input types", {
  expect_error(reconsi(Y = mat, x = x,
                          zVals = list(zValObs = rnorm(p),
                                         zValsPerm = matrix(rnorm(p*B), B, p)),
                          zValues = FALSE))
    #Zvalues supplied but not used
expect_error(fdrResLm = reconsi(mat, x, B = 5e1,
                                   test = function(x, y){
                                   c(summary(lm(y~x))$coef["x1","t value"],
                                     fit$df.residual)},
                                   quantileFun = function(t){
                                       pt(q = t[1], df = t[2])}))
#Wrong argument name for quantile function (t instead of q)
expect_error(reconsi(Y = mat, x = sample(x, p, replace = TRUE)))
#Array sizes do not correspond
expect_error(reconsi(Y = mat[,1], x = sample(x, p, replace = TRUE)))
#Vector rather than matrix supplied.
})

test_that("reconsi throws warnings where necessary", {
    expect_error(reconsi(Y = mat[,seq_len(25)],
                            x = sample(x, p, replace = TRUE)))
    #Few hypotheses
})
test_that("reconsi works smoothly when all is fine", {
    expect_error(reconsi(Y = mat, x = x), NA)
})
mat2 = mat
mat2[x==0,1] = 0
mat2[x==1,1] = 1
test_that("reconsi works smoothly even when some test statistics are NA", {
    expect_error(reconsi(Y = mat2, x = x, test = "t.test"), NA)
})
test_that("reconsi works when bootstrap is requested", {
              expect_error(reconsi(Y = mat, test = function(y, x, mu){
                  testRes = t.test(y, mu = mu)
                  c(testRes$statistic, testRes$parameter)}, argList = list(mu = 0), center = FALSE,
                  distFun = function(q){pt(q = q[1],
                                           df = q[2])}), NA)
    expect_error(reconsi(Y = mat, test = function(y, x){
        wilcox.test(y, mu = 0, exact = FALSE)$statistic
    }, distFun = "psignrank", testPargs = list(n = nrow(mat))), NA)
          })

#' A custom function to calculate the distribution function of the t-test
#'  statistic. For internal use only
#'@importFrom stats pt qt
#'@param q a vector with t-statistic and degrees of freedom
#'@return A value between 0 and 1, the evaluation of the cdf
ptEdit = function(q){
  pt(q = q[1], df = q[2])
}
#' A custom function to calculate the quantile function of the t-test
#'  statistic. For internal use only
#'@param p a vector with quantile and degrees of freedom
#'@return the corresponding quantile
qtEdit = function(p){
  qt(p = p[1], df = p[2])
}

#' Fast estimation of mean and standard deviation of a normal distrbution,
#' optionally with weights
#' @param y vector of observations
#' @param w optional weight vector
#' @param p The number of features
#'
#' @return A vector of length 2 with mean and standard deviation
#' @importFrom stats sd
#' @importFrom matrixStats weightedMean
estNormal = function(y, w = NULL, p = length(y)){
    if(is.null(w)){
        c(mean = mean(y, na.rm = TRUE), sd = sd(y, na.rm = TRUE))
    } else {
        wmean = weightedMean(y, w = w, na.rm = TRUE)
        c(mean = wmean, sd = sqrt(sum((y-wmean)^2*w, na.rm = TRUE)*1/(p-1)))
    }
}

context("input to testDAA")

test_that("testDAA deals with all zero rows", {
    n = 20; p = 50; B = 1e2
    mat = matrix(rpois(n*p, lambda = 3), n, p)
    x = sample(c(0,1), n , replace = TRUE)
    FC = rpois(n, 1000)
    mat[1,] = 0
    expect_no_error(testDAA(Y = mat, x = x, FC = FC))
})

---
output:
    rmarkdown::html_vignette:
        toc: true
        keep_md: true
vignette: >
    %\VignetteIndexEntry{Manual for the RCM pacakage}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

# reconsi package: vignette

\setcounter{tocdepth}{5}
\tableofcontents

# Introduction

The aim of this package is to improve simultaneous inference for correlated 
hypotheses using collapsed null distributions. These collapsed null distributions 
are estimated in an empirical Bayes framework through resampling. Wilcoxon rank sum test and two sample t-test
are natively implemented, but any other test can be used.

# Installation

```{r installBioConductor, eval = FALSE}
library(BiocManager)
BiocManager::install("reconsi")
```

```{r installAndLoadGitHub, eval = FALSE}
library(devtools)
install_github("CenterForStatistics-UGent/reconsi")
```

```{r loadReconsi}
suppressPackageStartupMessages(library(reconsi))
cat("reconsi package version", as.character(packageVersion("reconsi")), "\n")
```

## General use

We illustrate the general use of the package on a synthetic dataset. The default 
Wilcoxon rank-sum test is used.

```{r syntData}
#Create some synthetic data with 90% true null hypothesis
 p = 200; n = 50
x = rep(c(0,1), each = n/2)
 mat = cbind(
 matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
 matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
 )
 #Provide just the matrix and grouping factor, and test using the collapsed null
 fdrRes = reconsi(mat, x)
  #The estimated tail-area false discovery rates.
  estFdr = fdrRes$Fdr
```

The method provides an estimate of the proportion of true null hypothesis, which 
is close to the true 90%.

```{r p0}
fdrRes$p0
```

The result of the procedure can be represented graphically as follows:

```{r plotNull}
plotNull(fdrRes)
```

The approximate correlation matrix of test statistic in the univariate $h$ distribution can be visualized as follows:

```{r plotApproxCovar}
plotApproxCovar(fdrRes)
```

On the other hand, one can also view the full variance-covariance matrix of the test statistics as found by the resamples. Here we demonstrate how it can recover a block correlation structure in the data.

```{r plotCovar}
matBlock = mat
matBlock[x==0,] = matBlock[x==0,] + rep(c(-1,2), each = n*p/4)
fdrResBlock = reconsi(matBlock, x)
plotCovar(fdrResBlock)
```

It is also possible to provide a custom test function, which must accept at
least a 'y' response variable and a 'x' grouping factor. Additionally, quantile, distribution and density functions should be supplied for transformation through quantiles to z-values.

```{r customFunction}
 #With a custom function, here linear regression
fdrResLm = reconsi(mat, x, B = 5e1,
                      test = function(x, y){
fit = lm(y~x)
c(summary(fit)$coef["x","t value"], fit$df.residual)},
distFun = function(q){pt(q = q[1], df = q[2])})
```

This framework also accepts more than 2 groups, and additional covariates 
through the "argList" argument.

```{r customFunction2}
 #3 groups
 p = 100; n = 60
x = rep(c(0,1,2), each = n/3)
mu0 = 5
 mat = cbind(
 matrix(rnorm(n*p/10, mean = mu0+x),n,p/10), #DA
 matrix(rnorm(n*p*9/10, mean = mu0),n,p*9/10) #Non DA
 )
 #Provide an additional covariate through the 'argList' argument
 z = rpois(n , lambda = 2)
 fdrResLmZ = reconsi(mat, x, B = 5e1,
 test = function(x, y, z){
 fit = lm(y~x+z)
 c(summary(fit)$coef["x","t value"], fit$df.residual)},
distFun = function(q){pt(q = q[1], df = q[2])},
 argList = list(z = z))
```

If the null distribution of the test statistic is not known, it is also possbile
to execute the procedure on the scale of the original test statistics, rather 
than z-values by setting zValues = FALSE. This may be numerically less stable.

```{r kruskal}
fdrResKruskal = reconsi(mat, x, B = 5e1,
test = function(x, y){kruskal.test(y~x)$statistic}, zValues = FALSE)
```

Alternatively, the same resampling instances may be used to determine the marginal null distributions as to estimate the collapsed null distribution, by setting the "resamZvals" flag to true.

```{r resamZvals}
fdrResKruskalPerm = reconsi(mat, x, B = 5e1,
test = function(x, y){
 kruskal.test(y~x)$statistic}, resamZvals = TRUE)
```

When no grouping variable is available, one can perform a bootstrap as resampling procedure. This is achieved by simply not supplying a grouping variable "x". Here we perform a one sample Wilcoxon rank sum test for equality of the means to 0.

```{r bootstrap}
fdrResBootstrap = reconsi(Y = mat, B = 5e1, test = function(y, x, mu){
                                      testRes = t.test(y, mu = mu)
                                      c(testRes$statistic, testRes$parameter)}, argList = list(mu = mu0),
                                  distFun = function(q){pt(q = q[1],
                                                           df = q[2])})
```

## Case study

We illustrate the package using an application from microbiology. The species
composition of a community of microorganisms can be determined through 
sequencing. However, this only yields compositional information, and knowledge 
of the population size can be acquired by cell counting through flow cytometry.
Next, the obtained species compositions can multiplied by the total population 
size to yield approximate absolute cell counts per species. Evidently, this 
introduces strong correlation between the tests due to the common factor. In 
other words: random noise in the estimation of the total cell counts will affect 
all hypotheses. Therefore, we employ resampling to estimate the collapsed null distribution, that will account for this 
dependence.

The dataset used is taken from Vandeputte _et al._, 2017 (see [manuscript](https://www.ncbi.nlm.nih.gov/pubmed/29143816)), a study on gut microbiome 
in healthy and Crohn's disease patients. The test looks for differences in absolute abundance between healthy and diseased patients. It relies on the _phyloseq_ package, which is the preferred way to interact with our machinery for microbiome data.

```{r Vandeputte}
#The grouping and flow cytometry variables are present in the phyloseq object, they only need to be called by their name.
data("VandeputteData")
testVanDePutte = testDAA(Vandeputte, groupName = "Health.status", FCname = "absCountFrozen", B = 1e2L)
```

The estimated tail-area false discovery rates can then simply be extracted as

```{r vandeputteFdr}
FdrVDP = testVanDePutte$Fdr
quantile(FdrVDP)
```

# Session info

Finally all info on R and package version is shown

```{r sessionInfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_getApproxCovar.R
\name{getApproxCovar}
\alias{getApproxCovar}
\title{Obtain a null covariance matrix of binned test statistics}
\usage{
getApproxCovar(statsPerm, ...)
}
\arguments{
\item{statsPerm}{The pxB matrix of permutation z-values in the columns}

\item{...}{passed on to binStats}
}
\value{
The covariance matrix of binned z-values
}
\description{
Obtain a null covariance matrix of binned test statistics
}
\note{
This is not the covariance matrix of the p test statistic, nor of the
   data! It is an approximate covariance matrix of binned test statistics for
   visualization and diagnostic purposes.
}
\examples{
p = 200; n = 50; B = 5e1
x = rep(c(0,1), each = n/2)
mat = cbind(
matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
)
mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
fdrRes = reconsi(mat, x, B = B)
corMat = getApproxCovar(fdrRes$statsPerm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_ptEdit.R
\name{ptEdit}
\alias{ptEdit}
\title{A custom function to calculate the distribution function of the t-test
 statistic. For internal use only}
\usage{
ptEdit(q)
}
\arguments{
\item{q}{a vector with t-statistic and degrees of freedom}
}
\value{
A value between 0 and 1, the evaluation of the cdf
}
\description{
A custom function to calculate the distribution function of the t-test
 statistic. For internal use only
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_plotNull.R
\name{plotNull}
\alias{plotNull}
\title{Plot the obtained null distribution along with a histogram of observed test
statistics}
\usage{
plotNull(
  fit,
  lowColor = "yellow",
  highColor = "blue",
  idNull = NULL,
  nResampleCurves = length(fit$Weights),
  hSize = 0.5
)
}
\arguments{
\item{fit}{an object returned by the reconsi() (or testDAA()) function}

\item{lowColor, highColor}{The low and high ends of the colour scale}

\item{idNull}{indices of known null taxa}

\item{nResampleCurves}{The number of resampling null distributions to plot}

\item{hSize}{A double, the size of the line of the collapsed null estimate}
}
\value{
a ggplot2 plot object
}
\description{
Plot the obtained null distribution along with a histogram of observed test
statistics
}
\examples{
 p = 180; n = 50; B = 1e2
 #Low number of resamples keeps computation time down
 x = rep(c(0,1), each = n/2)
 mat = cbind(
 matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
 matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
 )
#Provide just the matrix and grouping factor, and test using the random null
fdrRes = reconsi(mat, x, B = B)
plotNull(fdrRes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_testDAA.R
\name{testDAA}
\alias{testDAA}
\alias{testDAA,phyloseq-method}
\alias{testDAA,matrix-method}
\title{A function to test for differential absolute abundance on a phyloseq object}
\usage{
testDAA(Y, ...)

\S4method{testDAA}{phyloseq}(Y, groupName, FCname, ...)

\S4method{testDAA}{matrix}(Y, FC, x, S = rowSums(Y), tieBreakRan = TRUE, assumeNormal = TRUE, ...)
}
\arguments{
\item{Y}{A phyloseq object, or a data matrix with samples in the rows and
OTUs in the columns}

\item{...}{passed on to the reconsi() function}

\item{groupName}{A character string, the name of a variable in physeq
indicating the grouping factor}

\item{FCname}{A character string, the name of a variable in physeq
containing the total cell count}

\item{FC}{a vector of length n with total flow cytometry cell counts}

\item{x}{a grouping factor of length n}

\item{S}{a vector of library sizes. Will be calculated if not provided}

\item{tieBreakRan}{A boolean, should ties be broken at random.}

\item{assumeNormal}{A boolean, should normality be assumed for the null distribution?}
}
\value{
See the reconsi() function
}
\description{
A function to test for differential absolute abundance on a phyloseq object
}
\examples{
#Test for phyloseq object
library(phyloseq)
data("VandeputteData")
VandeputtePruned = prune_samples(Vandeputte,
samples = sample_names(Vandeputte)[20:40])
testVanDePutte = testDAA(VandeputtePruned, "Health.status", "absCountFrozen",
B = 15)
#Test for matrix
testMat = testDAA(as(otu_table(VandeputtePruned), "matrix"),
get_variable(VandeputtePruned, "Health.status"),
get_variable(VandeputtePruned,"absCountFrozen"), B = 15)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_estP0.R
\name{estP0}
\alias{estP0}
\title{Estimate the fraction of true null hypotheses.}
\usage{
estP0(statObs, fitAll, z0quantRange, smooth.df, evalVal, assumeNormal)
}
\arguments{
\item{statObs}{A vector of observed z-values}

\item{fitAll}{the estimated normal null}

\item{z0quantRange}{a number of quantiles between 0 and 0.5}

\item{smooth.df}{degrees of freedom for the spline smoother}

\item{evalVal}{the value of q at which to evaluate the spline}

\item{assumeNormal}{A boolean, should normality be assumed for the null distribution?}
}
\value{
The estimated null fraction, the value of the spline evaluated
 at the first element of z0quantRange
}
\description{
Estimate the fraction of true null hypotheses.
}
\details{
A natural spline is used over a range of intervals.
Based on the qvalue::qvalue() function and
Storey and Tibshirani, 2003
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_data.R
\docType{data}
\name{Vandeputte}
\alias{Vandeputte}
\title{Microbiomes of Crohn's disease patients and healthy controls}
\format{
A phyloseq object with an OTU-table and sample data
\describe{
  \item{otu_table}{Count data matrix of 234 taxa in 135 samples}
  \item{sample_data}{Data frame of patient covariates}
}
}
\source{
\url{https://www.ncbi.nlm.nih.gov/pubmed/29143816}
}
\usage{
Vandeputte
}
\description{
Microbiome sequencing data of Crohn's disease patients,
and healthy controls, together with other baseline covariates.
Both sequencing and flow cytometry data are available.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_getG0.R
\name{getG0}
\alias{getG0}
\title{Obtain the consensus null}
\usage{
getG0(
  statObs,
  statsPerm,
  z0Quant,
  gridsize,
  maxIter,
  tol,
  estP0args,
  testPargs,
  B,
  p,
  pi0,
  assumeNormal,
  resamAssumeNormal
)
}
\arguments{
\item{statObs}{A vector of lenght p with observed test statistics}

\item{statsPerm}{A pxB matrix with permuation z-values}

\item{z0Quant}{a vector of length of quantiles defining the central part R
of the distribution. If a single number is supplied, then
(z0quant, 1-z0quant) will be used}

\item{gridsize}{An integer, the gridsize for the density estimation}

\item{maxIter}{An integer, the maximum number of iterations in determining R}

\item{tol}{The convergence tolerance.}

\item{estP0args}{A list of arguments passed on to the estP0args() function}

\item{testPargs}{A list of arguments passed on to quantileFun}

\item{B}{an integer, the number of permutations}

\item{p}{an integer, the number of hypotheses}

\item{pi0}{A known fraction of true null hypotheses}

\item{assumeNormal}{A boolean, should normality be assumed for the null distribution?}

\item{resamAssumeNormal}{A boolean, should normality be assumed for resampling dists}
}
\value{
A list with following entries
\item{PermDensFits}{The permutation density fits}
\item{zSeq}{The support of the kernel for density estimation}
\item{zValsDensObs}{The estimated densities of the observed z-values}
\item{convergence}{A boolean, has the algorithm converged?}
\item{weights}{Vector of length B with weights
   for the permutation distributions}
\item{fdr}{Estimated local false discovery rate along the support
   of the kernel}
\item{p0}{The estimated fraction of true null hypotheses}
\item{iter}{The number of iterations}
\item{fitAll}{The consensus null fit}
}
\description{
Obtain the consensus null
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_getC1prop.R
\name{getC1prop}
\alias{getC1prop}
\title{Find the dependence pat C1 of the approximate covariance matrix, and extract the ratio of the first eigenvalue to the sum of all positive eigenvalues}
\usage{
getC1prop(statsPerm, numEig = 1, ...)
}
\arguments{
\item{statsPerm}{Matrix of permuted test statistics}

\item{numEig}{An integer, number of first eigenvalues}

\item{...}{passed onto binStats}
}
\value{
A proportion indicating the ratio of the first eigenvalues to the sum of all eigenvalues
}
\description{
Find the dependence pat C1 of the approximate covariance matrix, and extract the ratio of the first eigenvalue to the sum of all positive eigenvalues
}
\examples{
p = 200; n = 50; B = 5e1
x = rep(c(0,1), each = n/2)
mat = cbind(
matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
)
mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
fdrRes = reconsi(mat, x, B = B)
getC1prop(fdrRes$statsPerm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_getApproxCovar.R
\name{binStats}
\alias{binStats}
\title{Bin the test statistic into equally sized bins}
\usage{
binStats(z, nBins = 82L, binEdges = c(-4.1, 4.1))
}
\arguments{
\item{z}{the matrix of permuted test statistics}

\item{nBins}{an integer, the number of bins}

\item{binEdges}{A vector of length 2 with the outer bin edges}
}
\value{
Matrix of binned test statistics
}
\description{
Bin the test statistic into equally sized bins
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_getTstat.R
\name{getTstat}
\alias{getTstat}
\title{A function to obtain a t-test statistic efficiently. For internal use only}
\usage{
getTstat(y1, y2, mm, nn)
}
\arguments{
\item{y1, y2}{vectors of obsereved values in the two groups}

\item{mm, nn}{number of observations in the corresponding groups}
}
\value{
A list with items
\item{tstat}{The t-test statistic}
\item{df}{The degrees of freedom (Welch approximation)}
}
\description{
A function to obtain a t-test statistic efficiently. For internal use only
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_quantCorrect.R
\name{quantCorrect}
\alias{quantCorrect}
\title{Correct quantiles by not returning 0 or 1}
\usage{
quantCorrect(quants)
}
\arguments{
\item{quants}{A vector of quantiles}
}
\value{
The same vector of quantiles but without 0 or 1 values
}
\description{
Correct quantiles by not returning 0 or 1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_getFdr.R
\name{getFdr}
\alias{getFdr}
\title{Calculate tail-area (Fdr) and local (fdr) false discovery rates,
   based on a certain null distribution}
\usage{
getFdr(
  statObs,
  fitAll,
  fdr,
  p,
  p0,
  zValsDensObs,
  smoothObs,
  assumeNormal,
  fitObs,
  ...
)
}
\arguments{
\item{statObs}{Vector of observed z-values}

\item{fitAll}{The parameters of the estimated random null}

\item{fdr}{local false discovery rate, already estimated}

\item{p}{the number of hypotheses}

\item{p0}{The estimated fraction of null hypotheses}

\item{zValsDensObs}{estimated densities of observed test statistics}

\item{smoothObs}{A boolean, should estimated observed densities of the test
statistics be used in estimating the Fdr}

\item{assumeNormal}{A boolean, should normality be assumed for the null distribution?}

\item{fitObs}{The kernel density estimate object of all test statistics}

\item{...}{more arguments, ignored}
}
\value{
A list with components
\item{Fdr}{Tail are false discovery rate}
\item{fdr}{Local false discovery rate}
}
\description{
Calculate tail-area (Fdr) and local (fdr) false discovery rates,
   based on a certain null distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_getTestStats.R
\name{getTestStats}
\alias{getTestStats}
\title{A function to calculate observed and permuation z-statistics
on a n-by-p matrix of observations}
\usage{
getTestStats(
  Y,
  center,
  test = "wilcox.test",
  x,
  B,
  argList,
  tieBreakRan,
  replace,
  scale
)
}
\arguments{
\item{Y}{The nxp data matrix}

\item{center}{a boolean, should data be centered prior to permuation}

\item{test}{A function name, possibly user defined. See details.}

\item{x}{A vector defining the groups. Will be coerced to factor.}

\item{B}{an integer, the number of permuations}

\item{argList}{A list of further arguments passed on to the test function}

\item{tieBreakRan}{A boolean, should ties of permutation test statistics
be broken randomly? If not, midranks are used}

\item{replace}{A boolean. If FALSE, samples are permuted
(resampled without replacement), if TRUE the samples are bootstrapped
(resampled with replacement)}

\item{scale}{a boolean, should data be scaled prior to resampling}
}
\value{
A list with components
\item{statObs}{A vector of length p of observed test statistics}
\item{statsPerm}{A p-by-B matrix of permutation test statistics}
\item{resamDesign}{The resampling design}
}
\description{
A function to calculate observed and permuation z-statistics
on a n-by-p matrix of observations
}
\details{
For test "wilcox.test" and "t.test",
fast custom implementations are used. Other functions can be supplied
but must accept a y outcome variable, a x as grouping variable, and possibly
a list of other arguments. It must return all arguments needed to evaluate
its quantile function if z-values are to be used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_rowMultiply.R
\name{rowMultiply}
\alias{rowMultiply}
\title{A function to efficiently row multiply a a-by-b matrix by a vector of
 length b. More memory intensive but that does not matter
  with given matrix sizes}
\usage{
rowMultiply(matrix, vector)
}
\arguments{
\item{matrix}{a numeric matrix of dimension a-by-b}

\item{vector}{a numeric vector of length b}
}
\value{
a matrix, row multplied by the vector
}
\description{
A function to efficiently row multiply a a-by-b matrix by a vector of
 length b. More memory intensive but that does not matter
  with given matrix sizes
}
\details{
t(t(matrix)*vector) but then faster
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_plotCovar.R
\name{plotCovar}
\alias{plotCovar}
\title{Plot an the corvariance matrix of the test statistics estimated through permutations}
\usage{
plotCovar(
  reconsiFit,
  col = colorRampPalette(c("yellow", "blue"))(12),
  xlab = "Test statistic index",
  ylab = xlab,
  ...
)
}
\arguments{
\item{reconsiFit}{The reconsi fit}

\item{col, xlab, ylab, ...}{A list of arguments for the image() function.}
}
\value{
invisible()
}
\description{
Plot an the corvariance matrix of the test statistics estimated through permutations
}
\details{
By default, yellow indicates negative correlaton between test statistics,
blue positive correlation
}
\note{
Note the difference with the plotApproxCovar function, where the
covariances between binned test statistics are shown to get an idea between
covariances between tail and center values of the univariate null distribution.
Here the covariance matrix between all test statistics is shown
}
\examples{
p = 200; n = 50; B = 5e1
x = rep(c(0,1), each = n/2)
mat = cbind(
matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
)
mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
fdrRes = reconsi(mat, x, B = B)
plotCovar(fdrRes)
}
\seealso{
\link{plotApproxCovar}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_ptEdit.R
\name{qtEdit}
\alias{qtEdit}
\title{A custom function to calculate the quantile function of the t-test
 statistic. For internal use only}
\usage{
qtEdit(p)
}
\arguments{
\item{p}{a vector with quantile and degrees of freedom}
}
\value{
the corresponding quantile
}
\description{
A custom function to calculate the quantile function of the t-test
 statistic. For internal use only
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_estNormal.R
\name{estNormal}
\alias{estNormal}
\title{Fast estimation of mean and standard deviation of a normal distrbution,
optionally with weights}
\usage{
estNormal(y, w = NULL, p = length(y))
}
\arguments{
\item{y}{vector of observations}

\item{w}{optional weight vector}

\item{p}{The number of features}
}
\value{
A vector of length 2 with mean and standard deviation
}
\description{
Fast estimation of mean and standard deviation of a normal distrbution,
optionally with weights
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_calcWeights.R
\name{calcWeights}
\alias{calcWeights}
\title{Obtain weights as posterior probabilities to calculate the consensus null}
\usage{
calcWeights(logDensPerm, fdr)
}
\arguments{
\item{logDensPerm}{A matrix with B rows  of logged density estimates of the B
permutation distributions, and p columns for the p observed test
statistics}

\item{fdr}{A vector of local false discovery rates for the observed tests
statistics of length p}
}
\value{
A vector of weights of length B
}
\description{
Obtain weights as posterior probabilities to calculate the consensus null
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_stabExp.R
\name{stabExp}
\alias{stabExp}
\title{A function to numerically stabilize an exponentiation. For internal use only}
\usage{
stabExp(exps)
}
\arguments{
\item{exps}{the vector to be exponentiated}
}
\value{
the vector with the maximum subtracted
}
\description{
A function to numerically stabilize an exponentiation. For internal use only
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_reconsi.R
\name{reconsi}
\alias{reconsi}
\title{Perform simultaneous inference through
collapsed resampling null distributions}
\usage{
reconsi(
  Y,
  x = NULL,
  B = 1000L,
  test = "wilcox.test",
  argList = list(),
  distFun = "pnorm",
  zValues = TRUE,
  testPargs = list(),
  z0Quant = 0.25,
  gridsize = 801L,
  maxIter = 100L,
  tol = 1e-06,
  zVals = NULL,
  center = FALSE,
  replace = is.null(x),
  assumeNormal = TRUE,
  estP0args = list(z0quantRange = seq(0.05, 0.45, 0.0125), smooth.df = 3, evalVal = 0.05),
  resamZvals = FALSE,
  smoothObs = TRUE,
  scale = FALSE,
  tieBreakRan = FALSE,
  pi0 = NULL,
  resamAssumeNormal = TRUE
)
}
\arguments{
\item{Y}{the matrix of sequencing counts}

\item{x}{a grouping factor. If provided, this grouping factor is permuted.
Otherwise a bootstrap procedure is performed}

\item{B}{the number of resampling instances}

\item{test}{Character string, giving the name of the function
to test for differential absolute abundance.
Must accept the formula interface. Features with tests resulting in
observed NA test statistics will be discarded}

\item{argList}{A list of arguments, passed on to the testing function}

\item{distFun}{the distribution function of the test statistic, or its name.
Must at least accept an argument named 'q', 'p' and 'x' respectively.}

\item{zValues}{A boolean, should test statistics be converted to z-values.
See details}

\item{testPargs}{A list of arguments passed on to distFun}

\item{z0Quant}{A vector of length 2 of quantiles of the null distribution,
in between which only null values are expected}

\item{gridsize}{The number of bins for the kernel density estimates}

\item{maxIter}{An integer, the maximum number of iterations in the estimation
of the null distribution}

\item{tol}{The tolerance for the infinity norm of the central borders
in the iterative procedure}

\item{zVals}{An optional list of observed (statObs) and
resampling (statsPerm) z-values. If supplied, the calculation
   of the observed and resampling test statistics is skipped
   and the algorithm proceeds with calculation
   of the consensus null distribution}

\item{center}{A boolean, should observations be centered
in each group prior to permuations? See details.}

\item{replace}{A boolean. Should resampling occur with replacement (boostrap)
or without replacement (permutation)}

\item{assumeNormal}{A boolean, should normality be assumed for the null distribution?}

\item{estP0args}{A list of arguments passed on to the estP0 function}

\item{resamZvals}{A boolean, should resampling rather than theoretical null
distributions be used?}

\item{smoothObs}{A boolean, should the fitted rather than estimated observed
distribution be used in the Fdr calculation?}

\item{scale}{a boolean, should data be scaled prior to resampling}

\item{tieBreakRan}{A boolean, should ties of resampling test statistics
be broken randomly? If not, midranks are used}

\item{pi0}{A known fraction of true null hypotheses. If provided,
the fraction of true null hypotheses will not be estimated.
Mainly for oracle purposes.}

\item{resamAssumeNormal}{A boolean, should normality be assumed for resampling dists}
}
\value{
A list with entries
\item{statsPerm}{Resampling Z-values}
\item{statObs}{Observed Z-values}
\item{distFun}{Density, distribution and
quantile function as given}
\item{testPargs}{Same as given}
\item{zValues}{A boolean, were z-values used?}
\item{resamZvals}{A boolean, were the resampling null distribution used?}
\item{cdfValObs}{Cumulative distribution function evaluation
of observed test statistics}
\item{p0estimated}{A boolean, was the fraction of true null hypotheses
estimated from the data?}
\item{Fdr,fdr}{Estimates of tail-area and local false discovery rates}
\item{p0}{Estimated or supplied fraction of true null hypotheses}
\item{iter}{Number of iterations executed}
\item{fitAll}{Mean and standard deviation estimated collapsed null}
\item{PermDensFits}{Mean and standard deviations of resamples}
\item{convergence}{A boolean, did the iterative algorithm converge?}
\item{zSeq}{Basis for the evaluation of the densities}
\item{weights}{weights of the resampling distributions}
\item{zValsDensObs}{Estimated overall densities, evaluated in zSeq}
}
\description{
Perform simultaneous inference through
collapsed resampling null distributions
}
\details{
Efron (2007) centers the observations in each group prior
 to permutation. As permutations will remove any genuine group differences
  anyway, we skip this step by default. If zValues = FALSE,
  the density is fitted on the original test statistics rather than converted
  to z-values. This unlocks the procedure for test statistics with unknown
  distributions, but may be numerically less stable.
}
\note{
Ideally, it would be better to only use unique resamples, to avoid
unnecesarry replicated calculations of the same test statistics. Yet this
issue is almost alwyas ignored in practice; as the sample size grows it also
becomes irrelevant. Notice also that this would require to place weights in
case of the bootstrap, as some bootstrap samples are more likely than others.
}
\examples{
#Important notice: low number of resamples B necessary to keep
# computation time down, but not recommended. Pray set B at 200 or higher.
p = 60; n = 20; B = 5e1
x = rep(c(0,1), each = n/2)
mat = cbind(
matrix(rnorm(n*p/10, mean = 5+x), n, p/10), #DA
matrix(rnorm(n*p*9/10, mean = 5), n, p*9/10) #Non DA
)
fdrRes = reconsi(mat, x, B = B)
fdrRes$p0
#Indeed close to 0.9
estFdr = fdrRes$Fdr
#The estimated tail-area false discovery rates.

#With another type of test. Need to supply quantile function in this case
fdrResLm = reconsi(mat, x, B = B,
test = function(x, y){
fit = lm(y~x)
c(summary(fit)$coef["x","t value"], fit$df.residual)},
distFun = function(q){pt(q = q[1], df = q[2])})

#With a test statistic without known null distribution(for small samples)
fdrResKruskal = reconsi(mat, x, B = B,
test = function(x, y){
kruskal.test(y~x)$statistic}, zValues = FALSE)

#Provide an additional covariate through the 'argList' argument
z = rpois(n , lambda = 2)
fdrResLmZ = reconsi(mat, x, B = B,
test = function(x, y, z){
fit = lm(y~x+z)
c(summary(fit)$coef["x","t value"], fit$df.residual)},
distFun = function(q){pt(q = q[1], df = q[2])},
argList = list(z = z))

#When nog grouping variable is provided, a bootstrap is performed
matBoot = cbind(
matrix(rnorm(n*p/10, mean = 1), n, p/10), #DA
matrix(rnorm(n*p*9/10, mean = 0), n, p*9/10) #Non DA
)
fdrResBoot = reconsi(matBoot, B = B,
test = function(y, x){testRes = t.test(y, mu = 0, var.equal = TRUE);
c(testRes$statistic, testRes$parameter)},
distFun = function(q){pt(q = q[1], df = q[2])},
center = TRUE, replace = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/F_plotApproxCovar.R
\name{plotApproxCovar}
\alias{plotApproxCovar}
\title{Plot an approximation of the correlation structure of the test statistics}
\usage{
plotApproxCovar(
  reconsiFit,
  col = colorRampPalette(c("yellow", "blue"))(12),
  x = seq(-4.2, 4.2, 0.1),
  y = seq(-4.2, 4.2, 0.1),
  xlab = "Z-values",
  ylab = "Z-values",
  nBins = 82L,
  binEdges = c(-4.1, 4.1),
  ...
)
}
\arguments{
\item{reconsiFit}{The reconsi fit}

\item{col, x, y, xlab, ylab, ...}{A list of arguments for the image() function.}

\item{nBins, binEdges}{passed on to the getApproxCovar function}
}
\value{
invisible()
}
\description{
Plot an approximation of the correlation structure of the test statistics
}
\details{
By default, yellow indicates negative correlaton between bin counts,
blue positive correlation
}
\note{
This is not the covariance matrix of the p test statistic, nor of the
   data! It is an approximate covariance matrix of binned test statistics for
   visualization purposes. See \link{plotCovar} for the full covariance matrix.
}
\examples{
p = 200; n = 50; B = 5e1
x = rep(c(0,1), each = n/2)
mat = cbind(
matrix(rnorm(n*p/10, mean = 5+x),n,p/10), #DA
matrix(rnorm(n*p*9/10, mean = 5),n,p*9/10) #Non DA
)
mat = mat = mat + rnorm(n, sd = 0.3) #Introduce some dependence
fdrRes = reconsi(mat, x, B = B)
plotApproxCovar(fdrRes)
}
\seealso{
\link{plotCovar}, \link{getApproxCovar}
}
